.\" file MASS/ppr.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN ppr
.TL
Projection Pursuit Regression
.DN
Fits a projection pursuit regression model to one or more responses,
using a modification of Friedman's `SMART' program.  The ridge
functions can be smooth functions fitted either by the non-parametric
super smoother or by smoothing splines.  Methods are supplied for
`print', `summary', `plot' and `predict'.
.CS
ppr(formula, data=sys.parent(), weights, subset,
    na.action, contrasts=NULL, ww=rep(1,q),
    nterms, max.terms=nterms, optlevel=2, 
    sm.method=c("supsmu", "spline", "gcvspline"),
    bass=0, span=0, df=5, gcvpen=1)
ppr(x, y, weights=rep(1,n), ww=rep(1,q), nterms,
    max.terms=nterms, optlevel=2,
    sm.method=c("supsmu", "spline", "gcvspline"),
    bass=0, span=0, df=5, gcvpen=1)
.RA
.AG formula
a formula specifying one or more response variables and the
explanatory variables.
.AG x
matrix of explanatory variables.  Rows represent observations, and
columns represent variables.  Missing values are not accepted. 
(For dispatch argument compatibility this is called `formula' in the code.) 
.AG y
matrix of response variables.  Rows represent observations, and
columns represent variables.  Missing values are not accepted. 
.AG nterms
number of terms to include in the final model.
.OA
.AG data
Data frame from which variables specified in `formula' are
preferentially to be taken.
.AG weights
a vector of weights for each case.
.AG ww
a vector of weights for each response, so the fit criterion is
the sum over case `i' and responses `j' of
`w_i ww_j (y_ij - fit_ij)^2' divided by the sum of `w_i'.
.AG subset
An index vector specifying the cases to be used in the training
sample.  (NOTE: If given, this argument must be named.)
.AG na.action
A function to specify the action to be taken if `NA's are found. The
default action is for the procedure to fail.  An alternative is
`na.omit', which leads to rejection of cases with missing values on
any required variable.  (NOTE: If given, this argument must be named.)
.AG contrasts
the contrasts to be used when any factor explanatory variables are coded.
.AG max.terms
maximum number of terms to choose from when building the model.
.AG optlevel
integer from 0 to 3 which determines the thoroughness of an
optimization routine in the SMART program. See the DETAILS section.
.AG sm.method
the method used for smoothing the ridge functions.  The default is to
use Friedman's super smoother `supsmu'.  The alternatives are to use
the smoothing spline code underlying `smooth.spline', either with a
specified (equivalent) degrees of freedom for each ridge function, or
to allow the smoothness to be chosen by GCV.
.AG bass
super smoother bass tone control used with automatic span selection
(see `supsmu'); the range of values is 0 to 10, with larger values
resulting in increased smoothing.
.AG span
super smoother span control (see `supsmu').  The default, `0',
results in automatic span selection by local cross validation. `span'
can also take a value in `(0, 1]'.
.AG df
if `sm.method' is `"spline"' specifies the smoothness of each ridge
term via the requested equivalent degrees of freedom.
.AG gcvpen
if `sm.method' is `"gcvspline"' this is the penalty used in the GCV
selection for each degree of freedom used.
.RT
A list with the following components, many of which are for use by the
method functions.
.RC call
the matched call
.RC p
the number of explanatory variables (after any coding)
.RC q
the number of response variables
.RC mu
the argument `nterms'
.RC ml
the argument `max.terms'
.RC gof
the overall residual (weighted) sum of squares for the selected model
.RC gofn
the overall residual (weighted) sum of squares against the number of
terms, up to `max.terms'.  Will be invalid (and zero) for less than
`nterms'.
.RC df
the argument `df'
.RC edf
if `sm.method' is `"spline"' or `"gcvspline"' the equivalent number of
degrees of freedom for each ridge term used.
.RC xnames
the names of the explanatory variables
.RC ynames
the names of the response variables
.RC alpha
a matrix of the projection directions, with a column for each ridge term
.RC beta
a matrix of the coefficients applied for each response to the ridge
terms: the rows are the responses and the columns the ridge terms
.RC yb
the weighted means of each response
.RC ys
the overall scale factor used: internally the responses are divided by
`ys' to have unit total weighted sum of squares.
.RC fitted.values
the fitted values, as a matrix if `q > 1'
.RC residuals
the residuals, as a matrix if `q > 1'
.RC smod
internal work array, which includes the ridge functions evaluated at
the training set points.
.DT
The basic method is given by Friedman (1984), and is essentially the
same code used by `ppreg'.  The code is extremely sensitive to the
compiler used, and this version uses double precision to try to reduce
differences between machines.  The differences are the ability to use
spline smoothers and the interface which should be much easier to use.
.PP
The algorithm first adds up to `max.terms' ridge terms one at a time;
it will use less if it is unable to find a term to add that makes
sufficient difference.  It then removes the least "important" term at
each step until `nterm' terms are left.  The levels of optimization
differ in how thoroughly the models are refitted during this process.
At level 0 the existing ridge terms are not refitted. At level 1 the
projection directions are not refitted, but the ridge functions and
the regression coefficients are.  Levels 2 and 3 refit all the terms
and are equivalent for one response; level 3 is more careful to
re-balance the contributions from each regressor at each step and so
is a little less likely to converge to a saddle point of the sum of
squares criterion.
.SH WARNING
There is a limit of 2500 rows when using spline-based methods.
.SH REFERENCES
Friedman, J. H. and Stuetzle, W. (1981)
Projection pursuit regression. 
.ul
Journal of the American Statistical Association
\fB76\fR, 817-823.
.PP
Friedman, J. H. (1984)
.ul
SMART User's Guide.
Laboratory for Computational Statistics, Stanford University Technical
Report No. 1.
.SA
`plot.ppr',  `ppreg',  `supsmu',  `smooth.spline'
.EX
# Note: your numerical values may differ
attach(rock)
rock1 <- data.frame(area=area/10000, peri=peri/10000,
		    shape=shape, perm=perm)
detach()
rock.ppr <- ppr(log(perm) ~ area1 + peri1 + shape,
		data=rock1, nterms=2, max.terms=5)
rock.ppr
Call:
ppr.formula(formula = log(perm) ~ area + peri + 
	shape, data = rock1, nterms = 2, max.terms = 5)

Goodness of fit:
 2 terms 3 terms 4 terms 5 terms 
 11.2196  7.1895  6.4565  5.8592

summary(rock.ppr)
Call:
ppr.formula(formula = log(perm) ~ area + peri + 
	shape, data = rock1, nterms = 2, max.terms = 5)

Goodness of fit:
 2 terms 3 terms 4 terms 5 terms 
 11.2196  7.1895  6.4565  5.8592

Projection direction vectors:
         term 1    term 2 
 area  0.319492  0.435617
 peri -0.945544 -0.866757
shape  0.062226  0.242839

Coefficients of ridge terms:
  term 1  term 2 
 1.00638 0.72915
par(mfrow=c(1,2), pty="s")
plot(rock.ppr)
plot(update(rock.ppr, bass=5))
plot(update(rock.ppr, sm.method="gcv", gcvpen=2))
.KW regression
.WR
